rem ---------------------------------------------------------------------------
rem  Trivadis AG, Baden-Dttwil/Basel/Bern/Lausanne/Zrich
rem               Dsseldorf/Frankfurt/Freiburg i.Br./Hamburg/Mnchen/Stuttgart
rem               Wien
rem               Switzerland/Germany/Austria Internet: http://www.trivadis.com
rem ---------------------------------------------------------------------------
rem ***************************************************************************
rem ****** sample script for course/seminar New Features 11g Release 1)  ******
rem ***************************************************************************
rem  Group/Privileges.: User
rem  Script-Name......: ex_bind_peeking.sql
rem  Developer........: Christian Antognini (christian.antognini@trivadis.com)
rem  Date.............: November 2007
rem  Version..........: Oracle Database 11g Release 1 - 11.1
rem  Description......: The script shows that the query optimizer automatically
rem                     recognize that an execution plan is not efficient 
rem                     for all bind variable values.
rem  Usage............: 
rem                     
rem  Input parameters.: 
rem  Output...........: 
rem  Called by........:
rem  Requirements.....: 
rem  Remarks..........: Before re-executing the script the shared pool 
rem                     should be flushed.
rem
rem ---------------------------------------------------------------------------
rem Changes:
rem DD.MM.YYYY Developer Change
rem ---------------------------------------------------------------------------
rem
rem ---------------------------------------------------------------------------

SET ECHO ON TERMOUT ON SERVEROUTPUT OFF FEEDBACK OFF PAGESIZE 100 LINESIZE 80

COLUMN PLAN_TABLE_OUTPUT FORMAT A70
COLUMN PREDICATE FORMAT A20
COLUMN EXECUTIONS FORMAT 9999 HEADING EXEC
COLUMN PEEKED FORMAT A6
COLUMN ROWS_PROCESSED FORMAT 9999 HEADING ROWS
COLUMN BUFFER_GETS FORMAT 9999 HEADING GETS
COLUMN CPU_TIME FORMAT 9999 HEADING CPU

DROP TABLE t PURGE;

VARIABLE n NUMBER

CLEAR SCREEN

REM
REM create test table and gather statistics
REM

CREATE TABLE t (id, n, pad, CONSTRAINT t_pk PRIMARY KEY (id))
AS 
SELECT rownum, CASE WHEN rownum>500 THEN 666 ELSE rownum END, rpad('*',50,'*') 
FROM dba_objects WHERE rownum <= 1000;

CREATE INDEX i ON t (n);

BEGIN
  dbms_stats.gather_table_stats(
    ownname => user, 
    tabname => 't', 
    estimate_percent => 100, 
    method_opt => 'for all columns size 254',
    cascade => TRUE
  );
END;
/

PAUSE

REM
REM show execution plan generated when peeked value is selective
REM

execute :n := 6

SELECT count(pad) FROM t WHERE n = :n;

SELECT * FROM table(dbms_xplan.display_cursor(NULL, NULL, 'basic'));

PAUSE

execute :n := 19

SELECT count(pad) FROM t WHERE n = :n;

SELECT * FROM table(dbms_xplan.display_cursor(NULL, NULL, 'basic'));

PAUSE

execute :n := 42

SELECT count(pad) FROM t WHERE n = :n;

SELECT * FROM table(dbms_xplan.display_cursor(NULL, NULL, 'basic'));

PAUSE

REM
REM show execution plan generated when peeked not selective,
REM notice that the previous plan is reused... but only
REM for 3 executions...
REM

execute :n := 666

SELECT count(pad) FROM t WHERE n = :n;

SELECT * FROM table(dbms_xplan.display_cursor(NULL, NULL, 'basic'));

PAUSE

execute :n := 666

SELECT count(pad) FROM t WHERE n = :n;

SELECT * FROM table(dbms_xplan.display_cursor(NULL, NULL, 'basic'));

PAUSE

execute :n := 666

SELECT count(pad) FROM t WHERE n = :n;

SELECT * FROM table(dbms_xplan.display_cursor(NULL, NULL, 'basic'));

PAUSE

execute :n := 666

SELECT count(pad) FROM t WHERE n = :n;

SELECT * FROM table(dbms_xplan.display_cursor(NULL, NULL, 'basic'));

PAUSE

REM
REM if a selective value is used a different plan is used
REM

execute :n := 42

SELECT count(pad) FROM t WHERE n = :n;

SELECT * FROM table(dbms_xplan.display_cursor(NULL, NULL, 'basic'));

PAUSE

REM
REM find out sql_id
REM

SELECT count(pad) FROM t WHERE n = :n;

SELECT prev_sql_id
FROM v$session
WHERE sid = sys_context('userenv','sid');

PAUSE

REM
REM let's have a look to the shared pool
REM

SELECT child_number, is_bind_sensitive, is_bind_aware, is_shareable
FROM v$sql
WHERE sql_id = '&sql_id';

PAUSE

SELECT child_number, predicate, range_id, low, high
FROM v$sql_cs_selectivity 
WHERE sql_id = '&sql_id'
ORDER BY child_number;

PAUSE

SELECT child_number, bucket_id, count
FROM v$sql_cs_histogram 
WHERE sql_id = '&sql_id'
ORDER BY child_number, bucket_id;

PAUSE

SELECT child_number, bind_set_hash_value, peeked, executions, rows_processed, buffer_gets, cpu_time
FROM v$sql_cs_statistics 
WHERE sql_id = '&sql_id'
ORDER BY child_number;

SET ECHO OFF
