rem ---------------------------------------------------------------------------
rem  Trivadis AG, Baden-Dttwil/Basel/Bern/Lausanne/Zrich
rem               Dsseldorf/Frankfurt/Freiburg i.Br./Hamburg/Mnchen/Stuttgart
rem               Wien
rem               Switzerland/Germany/Austria Internet: http://www.trivadis.com
rem ---------------------------------------------------------------------------
rem ***************************************************************************
rem ****** sample script for course/seminar New Features 11g Release 1)  ******
rem ***************************************************************************
rem  Group/Privileges.: User
rem  Script-Name......: ex_extended_statistics1.sql
rem  Developer........: Christian Antognini (christian.antognini@trivadis.com)
rem  Date.............: November 2007
rem  Version..........: Oracle Database 11g Release 1 - 11.1
rem  Description......: This script shows how to enhance the query optimizer 
rem                     estimations with extended statistics. The example uses
rem                     a predicate based on an expression.
rem  Usage............: 
rem                     
rem  Input parameters.: 
rem  Output...........: 
rem  Called by........:
rem  Requirements.....: 
rem  Remarks..........: 
rem
rem ---------------------------------------------------------------------------
rem Changes:
rem DD.MM.YYYY Developer Change
rem ---------------------------------------------------------------------------
rem
rem ---------------------------------------------------------------------------

SET ECHO ON TERMOUT ON SERVEROUTPUT OFF FEEDBACK OFF PAGESIZE 100 LINESIZE 80

DROP TABLE t PURGE;

COLUMN extension FORMAT A40

CLEAR SCREEN

REM
REM create test table with data and gather object statistics
REM

CREATE TABLE t 
AS
SELECT rownum AS id, 1+1/rownum AS n, rpad('*',50,'*') AS pad
FROM all_objects
WHERE rownum <= 1000;

BEGIN
  dbms_stats.gather_table_stats(
    ownname          => user,
    tabname          => 'T',
    estimate_percent => 100,
    method_opt       => 'for all columns size 254'
  );
END;
/

PAUSE

REM
REM the predicate "trunc(n) = 1" selects 999 rows
REM

SELECT count(*) FROM t WHERE trunc(n) = 1;

PAUSE

REM
REM the query optimizer does a completely wrong estimation
REM (10 instead of 999)
REM

EXPLAIN PLAN FOR SELECT count(*) FROM t WHERE trunc(n) = 1;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic rows'));

PAUSE

REM
REM create a extension to provide addition information
REM to the query optimizer
REM

SELECT dbms_stats.create_extended_stats(
         ownname   => user,
         tabname   => 'T',
         extension => '(trunc(n))'
       ) AS extension_name
FROM dual;

PAUSE

SELECT extension_name, extension
FROM user_stat_extensions
WHERE table_name = 'T';

PAUSE

REM
REM for the extension a regular virtual column is created
REM

SELECT column_name, virtual_column
FROM user_tab_cols
WHERE table_name = 'T';

PAUSE

SELECT DISTINCT &extension_name FROM t;

PAUSE

REM
REM when object statistics are gathered the virtual
REM column is processed as any other column... therefore,
REM column statistics are gathered for it as well
REM

BEGIN
  dbms_stats.gather_table_stats(
    ownname          => user,
    tabname          => 'T',
    estimate_percent => 100,
    method_opt       => 'for all columns size 254'
  );
END;
/

PAUSE

REM
REM thanks to the extension the query optimizer is able
REM to correctly estimate the cardinality
REM

EXPLAIN PLAN FOR SELECT count(*) FROM t WHERE trunc(n) = 1;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic rows'));

PAUSE

REM
REM drop the extension and check that the estimation are not good
REM

BEGIN
  dbms_stats.drop_extended_stats(
    ownname   => user,
    tabname   => 'T',
    extension => '(trunc(n))'
  );
END;
/

PAUSE

EXPLAIN PLAN FOR SELECT count(*) FROM t WHERE trunc(n) = 1;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic rows'));

REM
REM create a virtual column based on "trunc(n)" and gather
REM object statistics
REM

ALTER TABLE t ADD trunc_n AS (trunc(n)) VIRTUAL;

SELECT column_name, virtual_column
FROM user_tab_cols
WHERE table_name = 'T';

PAUSE

BEGIN
  dbms_stats.gather_table_stats(
    ownname          => user,
    tabname          => 'T',
    estimate_percent => 100,
    method_opt       => 'for all columns size 254'
  );
END;
/

REM
REM thanks to the virtual column the query optimizer is 
REM able to correctly estimate the cardinality
REM

EXPLAIN PLAN FOR SELECT count(*) FROM t WHERE trunc(n) = 1;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic rows'));

SET ECHO OFF
