rem ---------------------------------------------------------------------------
rem  Trivadis AG, Baden-Dttwil/Basel/Bern/Lausanne/Zrich
rem               Dsseldorf/Frankfurt/Freiburg i.Br./Hamburg/Mnchen/Stuttgart
rem               Wien
rem               Switzerland/Germany/Austria Internet: http://www.trivadis.com
rem ---------------------------------------------------------------------------
rem ***************************************************************************
rem ****** sample script for course/seminar New Features 11g Release 1)  ******
rem ***************************************************************************
rem  Group/Privileges.: User
rem  Script-Name......: ex_linguistic_like.sql
rem  Developer........: Christian Antognini (christian.atnognini@trivadis.com)
rem  Date.............: November 2007
rem  Version..........: Oracle Database 11g Release 1 - 11.1
rem  Description......: This script shows that a linguistic function-based
rem                     index can be used with the LIKE operator.
rem  Usage............: 
rem                     
rem  Input parameters.: 
rem  Output...........: 
rem  Called by........:
rem  Requirements.....: 
rem  Remarks..........: Run the script on both 10g and 11g to see the difference.
rem
rem ---------------------------------------------------------------------------
rem Changes:
rem DD.MM.YYYY Developer Change
rem ---------------------------------------------------------------------------
rem
rem ---------------------------------------------------------------------------

SET ECHO ON TERMOUT ON SERVEROUTPUT OFF FEEDBACK OFF PAGESIZE 100 LINESIZE 80

CLEAR SCREEN

DROP TABLE t PURGE;

CLEAR SCREEN

REM
REM display database version
REM

SELECT *
FROM v$version
WHERE rownum = 1;

PAUSE

REM
REM create a test table (incl. object statistics)
REM

CREATE TABLE t AS 
SELECT rownum AS id, rpad('*',25,'*') AS pad 
FROM all_objects
WHERE rownum <= 10;

execute dbms_stats.gather_table_stats(user,'t')

PAUSE

REM
REM create a linguistic (case insensitive) function-based index
REM

CREATE INDEX i ON t(nlssort(pad,'nls_sort=binary_ci'));

PAUSE

REM
REM the index cannot be used if the parameter NLS_COMP is set to BINARY
REM

ALTER SESSION SET nls_sort=binary_ci;
ALTER SESSION SET nls_comp=binary;

EXPLAIN PLAN FOR SELECT * FROM t WHERE pad = 'A';

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic'));

PAUSE

REM
REM the index can be used if the parameter NLS_COMP is set to LINGUISTIC
REM

ALTER SESSION SET nls_comp=linguistic;

EXPLAIN PLAN FOR SELECT * FROM t WHERE pad = 'A';

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic'));

PAUSE

REM
REM only as of 11g a LIKE can use the index
REM

EXPLAIN PLAN FOR SELECT * FROM t WHERE pad LIKE 'A%';

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic'));

SET ECHO OFF
