rem ---------------------------------------------------------------------------
rem  Trivadis AG, Baden-Dttwil/Basel/Bern/Lausanne/Zrich
rem               Dsseldorf/Frankfurt/Freiburg i.Br./Hamburg/Mnchen/Stuttgart
rem               Wien
rem               Switzerland/Germany/Austria Internet: http://www.trivadis.com
rem ---------------------------------------------------------------------------
rem ***************************************************************************
rem ****** sample script for course/seminar New Features 11g Release 1)  ******
rem ***************************************************************************
rem  Group/Privileges.: User
rem  Script-Name......: ex_partition_pruning.sql
rem  Developer........: MaM (maurice.mueller@trivadis.com)
rem  Date.............: October 2007
rem  Version..........: Oracle Database 11g Release 1 - 11.1
rem  Description......: Show different type of partition pruning, particularly 
rem                     the new join-filter pruning.
rem  Usage............: 
rem                     
rem  Input parameters.: 
rem  Output...........: 
rem  Called by........:
rem  Requirements.....: Run the script ex_partition_pruning_ctas.sql to create
rem                     the table used by the queries.
rem  Remarks..........: 
rem
rem ---------------------------------------------------------------------------
rem Changes:
rem DD.MM.YYYY Developer Change
rem ---------------------------------------------------------------------------
rem 13.11.2007 ChA       Formatting + changes some queries
rem ---------------------------------------------------------------------------

SET ECHO ON TERMOUT ON SERVEROUTPUT ON PAGESIZE 100 LINESIZE 80

CLEAR SCREEN

REM
REM join-filter pruning on range partitions
REM

EXPLAIN PLAN FOR
SELECT count(*) 
FROM f_sales_r s, d_times t
WHERE t.time_id = s.time_id
AND t.calendar_year = 2000;

SELECT * FROM table(dbms_xplan.display(NULL,NULL,'basic +partition'));

PAUSE

REM
REM join-filter pruning on range partitions 
REM subquery pruning on hash subpartitions
REM

EXPLAIN PLAN FOR
SELECT count(*) 
FROM f_sales_rh s, d_times t, d_countries co
WHERE t.time_id = s.TIME_ID
AND co.country_id = s.country_id
AND t.calendar_year = 2000
AND co.country_name = 'Germany';

SELECT * FROM table(dbms_xplan.display(NULL,NULL,'basic +partition'));

PAUSE

REM
REM join-filter pruning on range partitions 
REM join-filter pruning on hash subpartitions
REM

EXPLAIN PLAN FOR
SELECT count(*) 
FROM f_sales_rh s, d_times t, d_countries co
WHERE t.time_id = s.TIME_ID
AND co.country_id = s.country_id
AND t.calendar_year = 2000
AND co.country_region = 'Europe';

SELECT * FROM table(dbms_xplan.display(NULL,NULL,'basic +partition'));

PAUSE

REM
REM join-filter pruning on range partitions 
REM no pruning on hash supartitions
REM

EXPLAIN PLAN FOR
SELECT count(*) 
FROM f_sales_rh s, d_times t, d_customers cu
WHERE t.time_id = s.time_id
AND cu.country_id = s.country_id
AND cu.cust_id = s.cust_id
AND t.calendar_year = 2000
AND cu.cust_state_province = 'Baden-Wuerttemberg';

SELECT * FROM table(dbms_xplan.display(NULL,NULL,'basic +partition'));

PAUSE

REM
REM join-filter pruning on range partitions 
REM subquery pruning on list subpartitions
REM

EXPLAIN PLAN FOR
SELECT count(*) 
FROM f_sales_rl s, d_times t, d_countries co
WHERE t.time_id = s.time_id
AND co.country_id = s.country_id
AND t.calendar_year = 2000
AND co.country_name = 'Germany';

SELECT * FROM table(dbms_xplan.display(NULL,NULL,'basic +partition'));

PAUSE

REM
REM join-filter pruning on list partitions 
REM join-filter pruning on range subpartitions
REM

EXPLAIN PLAN FOR
SELECT count(*) 
FROM f_sales_lr s, d_times t, d_countries co
WHERE t.time_id = s.time_id
AND co.country_id = s.country_id
AND t.calendar_year = 2000
AND co.country_region = 'Europe';

SELECT * FROM table(dbms_xplan.display(NULL,NULL,'basic +partition'));

PAUSE

REM
REM join-filter pruning on hash partitions
REM

EXPLAIN PLAN FOR
SELECT count(*) 
FROM f_sales_h s, d_countries co
WHERE co.country_id = s.country_id
AND co.country_region = 'Europe';

SELECT * FROM table(dbms_xplan.display(NULL,NULL,'basic +partition'));

PAUSE

REM
REM join-filter pruning on range partitions 
REM subquery pruning on range subpartitions
REM

EXPLAIN PLAN FOR
SELECT count(*) 
FROM f_sales_rr s, d_times t1, d_times t2
WHERE t1.time_id = s.order_time_id
AND t1.calendar_year = 2000
AND t2.time_id = s.pay_time_id
AND t2.calendar_year = 2001;

SELECT * FROM table(dbms_xplan.display(NULL,NULL,'basic +partition'));

PAUSE

REM
REM join-filter pruning on list partitions 
REM subquery pruning on range subpartitions
REM

EXPLAIN PLAN FOR
SELECT count(*)
from f_sales_lr s, d_times t, d_countries co
where t.time_id = s.time_id
and co.country_id = s.country_id
and t.calendar_year = 2000
and co.country_name = 'Germany';

SELECT * FROM table(dbms_xplan.display(NULL,NULL,'basic +partition'));

PAUSE

REM
REM join-filter pruning on list partitions 
REM join-filter pruning on range subpartitions
REM

EXPLAIN PLAN FOR
SELECT count(*) 
FROM f_sales_lr s, d_times t, d_customers cu
WHERE t.time_id = s.time_id
AND cu.country_id = s.country_id
AND cu.cust_id = s.cust_id
AND t.calendar_year = 2000
AND cu.cust_state_province = 'Baden-Wuerttemberg';

SELECT * FROM table(dbms_xplan.display(NULL,NULL,'basic +partition'));

PAUSE

REM
REM Cartesian join on dimensions
REM join-filter pruning on list partitions 
REM no pruning on hash supartitions
REM

EXPLAIN PLAN FOR
SELECT count(*) 
FROM f_sales_lh s, d_countries co, d_customers cu
WHERE co.country_id = s.country_id
AND cu.cust_id = s.cust_id
AND co.country_name = 'Germany'
AND cu.cust_state_province = 'Baden-Wuerttemberg';

SELECT * FROM table(dbms_xplan.display(NULL,NULL,'basic +partition'));

PAUSE

REM
REM Cartesian join on dimensions
REM pruning on list partitions and subpartitions
REM

EXPLAIN PLAN FOR
SELECT count(*) 
FROM f_sales_ll s, d_countries co1, d_countries co2
WHERE co1.country_id = s.in_country_id
AND co1.country_name = 'Germany'
AND co2.country_id = s.out_country_id
AND co2.country_name = 'France';

SELECT * FROM table(dbms_xplan.display(NULL,NULL,'basic +partition'));

PAUSE

REM
REM join-filter pruning on list partitions and subpartitions
REM

EXPLAIN PLAN FOR
SELECT count(*)
FROM f_sales_ll s, d_countries co1, d_countries co2
WHERE co1.country_id = s.in_country_id
AND co1.country_iso_code IN ('DE','FR','IT','ES','US')
AND co2.country_id = s.out_country_id
AND co2.country_iso_code IN ('DE','FR','IT','ES','US');

SELECT * FROM table(dbms_xplan.display(NULL,NULL,'basic +partition'));

SET ECHO OFF
