rem ---------------------------------------------------------------------------
rem  Trivadis AG, Baden-Dttwil/Basel/Bern/Lausanne/Zrich
rem               Dsseldorf/Frankfurt/Freiburg i.Br./Hamburg/Mnchen/Stuttgart
rem               Wien
rem               Switzerland/Germany/Austria Internet: http://www.trivadis.com
rem ---------------------------------------------------------------------------
rem ***************************************************************************
rem ****** sample script for course/seminar New Features 11g Release 1)  ******
rem ***************************************************************************
rem  Group/Privileges.: User
rem  Script-Name......: ex_pending_object_statistics.sql
rem  Developer........: Christian Antognini (christian.antognini@trivadis.com)
rem  Date.............: November 2007
rem  Version..........: Oracle Database 11g Release 1 - 11.1
rem  Description......: The script shows how to gather, use and publish pending
rem                     statistics.
rem  Usage............: 
rem                     
rem  Input parameters.: 
rem  Output...........: 
rem  Called by........:
rem  Requirements.....: 
rem  Remarks..........: 
rem
rem ---------------------------------------------------------------------------
rem Changes:
rem DD.MM.YYYY Developer Change
rem ---------------------------------------------------------------------------
rem 
rem ---------------------------------------------------------------------------

SET ECHO ON TERMOUT ON SERVEROUTPUT OFF FEEDBACK OFF PAGESIZE 100 LINESIZE 80

DROP TABLE t PURGE;

CLEAR SCREEN

REM
REM create test table, gather object statistics and check their utilization
REM

execute dbms_random.seed(0)

CREATE TABLE t
AS
SELECT rownum AS id,
       dbms_random.string('p',250) AS pad
FROM all_objects
WHERE rownum <= 1000
ORDER BY dbms_random.value;

PAUSE

BEGIN
  dbms_stats.set_table_prefs(user,'t','publish','true');
  dbms_stats.gather_table_stats(
    ownname          => user,
    tabname          => 'T',
    estimate_percent => 100,
    method_opt       => 'for all columns size 1',
    cascade          => TRUE
  );
END;
/

SELECT num_rows FROM user_tables WHERE table_name = 'T';

SELECT num_rows FROM user_tab_pending_stats WHERE table_name = 'T';

PAUSE

EXPLAIN PLAN FOR SELECT * FROM t;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic rows'));

PAUSE

REM
REM insert more data into the test table and gather pending statistics
REM

INSERT INTO t SELECT 1000+id, pad FROM t;

COMMIT;

PAUSE

BEGIN
  dbms_stats.set_table_prefs(user,'t','publish','false');
  dbms_stats.gather_table_stats(
    ownname          => user,
    tabname          => 'T',
    estimate_percent => 100,
    method_opt       => 'for all columns size 1',
    cascade          => TRUE
  );
  dbms_stats.set_table_prefs(user,'t','publish','true');
END;
/

SELECT num_rows FROM user_tables WHERE table_name = 'T';

SELECT num_rows FROM user_tab_pending_stats WHERE table_name = 'T';

PAUSE

REM
REM use current object statistics
REM

ALTER SESSION SET optimizer_use_pending_statistics = FALSE;

EXPLAIN PLAN FOR SELECT * FROM t;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic rows'));

PAUSE

REM
REM use pending object statistics
REM

ALTER SESSION SET optimizer_use_pending_statistics = TRUE;

EXPLAIN PLAN FOR SELECT * FROM t;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic rows'));

PAUSE 

REM
REM publish pending statistics and check their utilization
REM 

SELECT num_rows FROM user_tables WHERE table_name = 'T';

execute dbms_stats.publish_pending_stats(ownname => user, tabname => 'T');

SELECT num_rows FROM user_tables WHERE table_name = 'T';

ALTER SESSION SET optimizer_use_pending_statistics = FALSE;

PAUSE

EXPLAIN PLAN FOR SELECT * FROM t;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic rows'));

SET ECHO OFF
